
package stocknav.actions;

import stocknav.commandes.*;

import com.expershop.esapi.Session;
import com.expershop.esapi.Action;
import com.expershop.esapi.ServletRequest;

import java.util.Enumeration;
import java.util.Hashtable;

/**
 * Mise a jour des commandes
 * Input parameters:
 * Numcde: le numero de commande
 * Agence: le code agence
 * Pour chaque ligne de la commande:
 * QTY_<Numlig>  (quantite)
 * REMISE_<Numlig>  (remise)
 * PRIXU_<Numlig>  (prix unitaire)
 * DEL_<Numlig> (OPTIONAL, "y" means mark for delete)
 * Ajout (Facultatif: ajout article)
 * Qte (Facultatif: quantite pour ajout article)
 * Remiseg (remise globale)
 * Datelivr (date livraison souhaitee)
 */
public class MajCommande extends Action {

  private Hashtable remises_ = new Hashtable();

  public boolean serverSide() { return false; }

  public boolean service(Session session, ServletRequest req,
  Object info) throws Exception {

    int numcde = req.getIntParam("Numcde", -1);
    if(numcde == -1) {
      req.addParameter("MissingField", "Numcde");
      return false;
    }
    int agence = req.getIntParam("Agence", -1);
    if(agence == -1) {
      req.addParameter("MissingField", "Agence");
      return false;
    }

    Commande cmde = new Commande(session.getJdbcConnection(), numcde, agence);

    int ajout = req.getIntParam("Ajout", -1);
    if(ajout >= 0) {
      cmde.ajoutArticle(ajout, req.getDoubleParam("Qte", 1));
    }

    Enumeration names = req.getParameterNames();
    while(names.hasMoreElements()) {

      String name = names.nextElement().toString();

      if(name.startsWith("DEL_")) {
        String nl = name.substring("DEL_".length());
        String del = req.getParameter(name);
        try {
          int numlig = Integer.parseInt(nl); 
          if(del.equalsIgnoreCase("1")) {
            cmde.setAsupp(numlig, true);
          } else {
            cmde.setAsupp(numlig, false);
          }
        } catch(NumberFormatException e) {
        }

      // Verifier si une ligne precedemment a supprimer a ete deselectionnee
      // Probleme: si la checkbox est deselectionnee, pas de valeur transmise!
      // Donc necessite de transmettre un champ "hidden" qui demande de verifier
      } else if(name.startsWith("VDEL_")) {
        String nl = name.substring("VDEL_".length());
        String as = req.getParameter("DEL_" + nl);
        if(as == null) { // La checkbox a ete deselectionnee!
          try {
            int numlig = Integer.parseInt(nl);
            cmde.setAsupp(numlig, false);
          } catch(NumberFormatException e) {
          }
        }

      } else if(name.startsWith("QTY_")) {
        String nl = name.substring("QTY_".length());
        String qt = req.getParameter(name);
        try {
          double qte = Double.valueOf(qt).doubleValue();
          int numlig = Integer.parseInt(nl);

          cmde.setQuantite(numlig, qte);
        } catch(NumberFormatException e) {
        }

      } else if(name.startsWith("REMISE_")) {
        String nl = name.substring("REMISE_".length());
        String rem = req.getParameter(name);
        try {
          double r = Double.valueOf(rem).doubleValue();
          int numlig = Integer.parseInt(nl);

          if(r != cmde.getRemise(numlig)) {
            cmde.setRemise(numlig, r);
            remises_.put(nl, new Double(r)); // Garder trace de la remise
          }
        } catch(NumberFormatException e) {
        }

      }
    }

    // Les prix unitaires sont traites APRES les remises ligne, et ne sont
    // modifies que si aucune remise n'a ete appliquee.
    names = req.getParameterNames();
    while(names.hasMoreElements()) {

      String name = names.nextElement().toString();
      if(name.startsWith("PRIXU_")) {
        String nl = name.substring("PRIXU_".length());
        String pu = req.getParameter(name);
        try {
          double p = Double.valueOf(pu).doubleValue();
          int numlig = Integer.parseInt(nl);

          // Si une remise a deja ete appliquee, ne rien faire!
          if(remises_.get(nl) == null) {
            cmde.setPrixu(numlig, p);
          }
        } catch(NumberFormatException e) {
        }
      }
    }

    // Gestion de la remise globale: elle est appliquee A LA FIN,
    // apres d'eventuelles remises ou modifications de prix unitaire ligne
    double remiseg = req.getDoubleParam("Remiseg", 0);
    if(remiseg > 0) {
      cmde.setRemiseGlobale(remiseg); 
    }

    String datelivr = req.getParameter("Datelivr");
    if(datelivr != null) {
      cmde.setDateLivr(datelivr);
    }
    String commentaire = req.getParameter("Commentaire");
    if(commentaire != null) {
      cmde.setCommentaire(commentaire);
    }  
    cmde.saveChanges(); // Recalculer la table d'en-tete
    return true;
  }

};

